# ruff: noqa
# register all models in the app so they can be used with apps.get_model()
from canvas_sdk.v1.data.command import Command
from canvas_sdk.v1.data.condition import Condition
from canvas_sdk.v1.data.note import Note
from canvas_sdk.v1.data.patient import Patient
from canvas_sdk.v1.data.user import CanvasUser
