from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class EffectType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    UNKNOWN_EFFECT: _ClassVar[EffectType]
    LOG: _ClassVar[EffectType]
    ADD_PLAN_COMMAND: _ClassVar[EffectType]
    AUTOCOMPLETE_SEARCH_RESULTS: _ClassVar[EffectType]
    ADD_BANNER_ALERT: _ClassVar[EffectType]
    REMOVE_BANNER_ALERT: _ClassVar[EffectType]
    ORIGINATE_ASSESS_COMMAND: _ClassVar[EffectType]
    EDIT_ASSESS_COMMAND: _ClassVar[EffectType]
    DELETE_ASSESS_COMMAND: _ClassVar[EffectType]
    COMMIT_ASSESS_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_ASSESS_COMMAND: _ClassVar[EffectType]
    ORIGINATE_DIAGNOSE_COMMAND: _ClassVar[EffectType]
    EDIT_DIAGNOSE_COMMAND: _ClassVar[EffectType]
    DELETE_DIAGNOSE_COMMAND: _ClassVar[EffectType]
    COMMIT_DIAGNOSE_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_DIAGNOSE_COMMAND: _ClassVar[EffectType]
    ORIGINATE_GOAL_COMMAND: _ClassVar[EffectType]
    EDIT_GOAL_COMMAND: _ClassVar[EffectType]
    DELETE_GOAL_COMMAND: _ClassVar[EffectType]
    COMMIT_GOAL_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_GOAL_COMMAND: _ClassVar[EffectType]
    ORIGINATE_HPI_COMMAND: _ClassVar[EffectType]
    EDIT_HPI_COMMAND: _ClassVar[EffectType]
    DELETE_HPI_COMMAND: _ClassVar[EffectType]
    COMMIT_HPI_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_HPI_COMMAND: _ClassVar[EffectType]
    ORIGINATE_MEDICATION_STATEMENT_COMMAND: _ClassVar[EffectType]
    EDIT_MEDICATION_STATEMENT_COMMAND: _ClassVar[EffectType]
    DELETE_MEDICATION_STATEMENT_COMMAND: _ClassVar[EffectType]
    COMMIT_MEDICATION_STATEMENT_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_MEDICATION_STATEMENT_COMMAND: _ClassVar[EffectType]
    ORIGINATE_PLAN_COMMAND: _ClassVar[EffectType]
    EDIT_PLAN_COMMAND: _ClassVar[EffectType]
    DELETE_PLAN_COMMAND: _ClassVar[EffectType]
    COMMIT_PLAN_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_PLAN_COMMAND: _ClassVar[EffectType]
    ORIGINATE_PRESCRIBE_COMMAND: _ClassVar[EffectType]
    EDIT_PRESCRIBE_COMMAND: _ClassVar[EffectType]
    DELETE_PRESCRIBE_COMMAND: _ClassVar[EffectType]
    COMMIT_PRESCRIBE_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_PRESCRIBE_COMMAND: _ClassVar[EffectType]
    ORIGINATE_QUESTIONNAIRE_COMMAND: _ClassVar[EffectType]
    EDIT_QUESTIONNAIRE_COMMAND: _ClassVar[EffectType]
    DELETE_QUESTIONNAIRE_COMMAND: _ClassVar[EffectType]
    COMMIT_QUESTIONNAIRE_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_QUESTIONNAIRE_COMMAND: _ClassVar[EffectType]
    ORIGINATE_REASON_FOR_VISIT_COMMAND: _ClassVar[EffectType]
    EDIT_REASON_FOR_VISIT_COMMAND: _ClassVar[EffectType]
    DELETE_REASON_FOR_VISIT_COMMAND: _ClassVar[EffectType]
    COMMIT_REASON_FOR_VISIT_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_REASON_FOR_VISIT_COMMAND: _ClassVar[EffectType]
    ORIGINATE_STOP_MEDICATION_COMMAND: _ClassVar[EffectType]
    EDIT_STOP_MEDICATION_COMMAND: _ClassVar[EffectType]
    DELETE_STOP_MEDICATION_COMMAND: _ClassVar[EffectType]
    COMMIT_STOP_MEDICATION_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_STOP_MEDICATION_COMMAND: _ClassVar[EffectType]
    ORIGINATE_UPDATE_GOAL_COMMAND: _ClassVar[EffectType]
    EDIT_UPDATE_GOAL_COMMAND: _ClassVar[EffectType]
    DELETE_UPDATE_GOAL_COMMAND: _ClassVar[EffectType]
    COMMIT_UPDATE_GOAL_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_UPDATE_GOAL_COMMAND: _ClassVar[EffectType]
    ORIGINATE_PERFORM_COMMAND: _ClassVar[EffectType]
    EDIT_PERFORM_COMMAND: _ClassVar[EffectType]
    DELETE_PERFORM_COMMAND: _ClassVar[EffectType]
    COMMIT_PERFORM_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_PERFORM_COMMAND: _ClassVar[EffectType]
    ORIGINATE_INSTRUCT_COMMAND: _ClassVar[EffectType]
    EDIT_INSTRUCT_COMMAND: _ClassVar[EffectType]
    DELETE_INSTRUCT_COMMAND: _ClassVar[EffectType]
    COMMIT_INSTRUCT_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_INSTRUCT_COMMAND: _ClassVar[EffectType]
    ORIGINATE_LAB_ORDER_COMMAND: _ClassVar[EffectType]
    EDIT_LAB_ORDER_COMMAND: _ClassVar[EffectType]
    DELETE_LAB_ORDER_COMMAND: _ClassVar[EffectType]
    COMMIT_LAB_ORDER_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_LAB_ORDER_COMMAND: _ClassVar[EffectType]
    ORIGINATE_FAMILY_HISTORY_COMMAND: _ClassVar[EffectType]
    EDIT_FAMILY_HISTORY_COMMAND: _ClassVar[EffectType]
    DELETE_FAMILY_HISTORY_COMMAND: _ClassVar[EffectType]
    COMMIT_FAMILY_HISTORY_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_FAMILY_HISTORY_COMMAND: _ClassVar[EffectType]
    ORIGINATE_ALLERGY_COMMAND: _ClassVar[EffectType]
    EDIT_ALLERGY_COMMAND: _ClassVar[EffectType]
    DELETE_ALLERGY_COMMAND: _ClassVar[EffectType]
    COMMIT_ALLERGY_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_ALLERGY_COMMAND: _ClassVar[EffectType]
    ORIGINATE_REMOVE_ALLERGY_COMMAND: _ClassVar[EffectType]
    EDIT_REMOVE_ALLERGY_COMMAND: _ClassVar[EffectType]
    DELETE_REMOVE_ALLERGY_COMMAND: _ClassVar[EffectType]
    COMMIT_REMOVE_ALLERGY_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_REMOVE_ALLERGY_COMMAND: _ClassVar[EffectType]
    ORIGINATE_SURGICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    EDIT_SURGICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    DELETE_SURGICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    COMMIT_SURGICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_SURGICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    CREATE_TASK: _ClassVar[EffectType]
    UPDATE_TASK: _ClassVar[EffectType]
    CREATE_TASK_COMMENT: _ClassVar[EffectType]
    ORIGINATE_MEDICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    EDIT_MEDICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    DELETE_MEDICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    COMMIT_MEDICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_MEDICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    ADD_OR_UPDATE_PROTOCOL_CARD: _ClassVar[EffectType]
    ORIGINATE_TASK_COMMAND: _ClassVar[EffectType]
    EDIT_TASK_COMMAND: _ClassVar[EffectType]
    DELETE_TASK_COMMAND: _ClassVar[EffectType]
    COMMIT_TASK_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_TASK_COMMAND: _ClassVar[EffectType]
    ORIGINATE_REFILL_COMMAND: _ClassVar[EffectType]
    EDIT_REFILL_COMMAND: _ClassVar[EffectType]
    DELETE_REFILL_COMMAND: _ClassVar[EffectType]
    COMMIT_REFILL_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_REFILL_COMMAND: _ClassVar[EffectType]
    ORIGINATE_VITALS_COMMAND: _ClassVar[EffectType]
    EDIT_VITALS_COMMAND: _ClassVar[EffectType]
    DELETE_VITALS_COMMAND: _ClassVar[EffectType]
    COMMIT_VITALS_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_VITALS_COMMAND: _ClassVar[EffectType]
    ORIGINATE_UPDATE_DIAGNOSIS_COMMAND: _ClassVar[EffectType]
    EDIT_UPDATE_DIAGNOSIS_COMMAND: _ClassVar[EffectType]
    DELETE_UPDATE_DIAGNOSIS_COMMAND: _ClassVar[EffectType]
    COMMIT_UPDATE_DIAGNOSIS_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_UPDATE_DIAGNOSIS_COMMAND: _ClassVar[EffectType]
    ORIGINATE_CLOSE_GOAL_COMMAND: _ClassVar[EffectType]
    EDIT_CLOSE_GOAL_COMMAND: _ClassVar[EffectType]
    DELETE_CLOSE_GOAL_COMMAND: _ClassVar[EffectType]
    COMMIT_CLOSE_GOAL_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_CLOSE_GOAL_COMMAND: _ClassVar[EffectType]
    ANNOTATE_PATIENT_CHART_CONDITION_RESULTS: _ClassVar[EffectType]
    ANNOTATE_CLAIM_CONDITION_RESULTS: _ClassVar[EffectType]
    SHOW_PATIENT_CHART_SUMMARY_SECTIONS: _ClassVar[EffectType]
    SHOW_PATIENT_PROFILE_SECTIONS: _ClassVar[EffectType]
    PATIENT_PROFILE__ADD_PHARMACY__POST_SEARCH_RESULTS: _ClassVar[EffectType]
    SEND_SURESCRIPTS_ELIGIBILITY_REQUEST: _ClassVar[EffectType]
    SEND_SURESCRIPTS_MEDICATION_HISTORY_REQUEST: _ClassVar[EffectType]
    SEND_SURESCRIPTS_BENEFITS_REQUEST: _ClassVar[EffectType]
    ORIGINATE_EXAM_COMMAND: _ClassVar[EffectType]
    EDIT_EXAM_COMMAND: _ClassVar[EffectType]
    DELETE_EXAM_COMMAND: _ClassVar[EffectType]
    COMMIT_EXAM_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_EXAM_COMMAND: _ClassVar[EffectType]
    ORIGINATE_ROS_COMMAND: _ClassVar[EffectType]
    EDIT_ROS_COMMAND: _ClassVar[EffectType]
    DELETE_ROS_COMMAND: _ClassVar[EffectType]
    COMMIT_ROS_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_ROS_COMMAND: _ClassVar[EffectType]
    ORIGINATE_STRUCTURED_ASSESSMENT_COMMAND: _ClassVar[EffectType]
    EDIT_STRUCTURED_ASSESSMENT_COMMAND: _ClassVar[EffectType]
    DELETE_STRUCTURED_ASSESSMENT_COMMAND: _ClassVar[EffectType]
    COMMIT_STRUCTURED_ASSESSMENT_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_STRUCTURED_ASSESSMENT_COMMAND: _ClassVar[EffectType]
    SHOW_ACTION_BUTTON: _ClassVar[EffectType]
    PATIENT_PORTAL__INTAKE_FORM_RESULTS: _ClassVar[EffectType]
    LAUNCH_MODAL: _ClassVar[EffectType]
UNKNOWN_EFFECT: EffectType
LOG: EffectType
ADD_PLAN_COMMAND: EffectType
AUTOCOMPLETE_SEARCH_RESULTS: EffectType
ADD_BANNER_ALERT: EffectType
REMOVE_BANNER_ALERT: EffectType
ORIGINATE_ASSESS_COMMAND: EffectType
EDIT_ASSESS_COMMAND: EffectType
DELETE_ASSESS_COMMAND: EffectType
COMMIT_ASSESS_COMMAND: EffectType
ENTER_IN_ERROR_ASSESS_COMMAND: EffectType
ORIGINATE_DIAGNOSE_COMMAND: EffectType
EDIT_DIAGNOSE_COMMAND: EffectType
DELETE_DIAGNOSE_COMMAND: EffectType
COMMIT_DIAGNOSE_COMMAND: EffectType
ENTER_IN_ERROR_DIAGNOSE_COMMAND: EffectType
ORIGINATE_GOAL_COMMAND: EffectType
EDIT_GOAL_COMMAND: EffectType
DELETE_GOAL_COMMAND: EffectType
COMMIT_GOAL_COMMAND: EffectType
ENTER_IN_ERROR_GOAL_COMMAND: EffectType
ORIGINATE_HPI_COMMAND: EffectType
EDIT_HPI_COMMAND: EffectType
DELETE_HPI_COMMAND: EffectType
COMMIT_HPI_COMMAND: EffectType
ENTER_IN_ERROR_HPI_COMMAND: EffectType
ORIGINATE_MEDICATION_STATEMENT_COMMAND: EffectType
EDIT_MEDICATION_STATEMENT_COMMAND: EffectType
DELETE_MEDICATION_STATEMENT_COMMAND: EffectType
COMMIT_MEDICATION_STATEMENT_COMMAND: EffectType
ENTER_IN_ERROR_MEDICATION_STATEMENT_COMMAND: EffectType
ORIGINATE_PLAN_COMMAND: EffectType
EDIT_PLAN_COMMAND: EffectType
DELETE_PLAN_COMMAND: EffectType
COMMIT_PLAN_COMMAND: EffectType
ENTER_IN_ERROR_PLAN_COMMAND: EffectType
ORIGINATE_PRESCRIBE_COMMAND: EffectType
EDIT_PRESCRIBE_COMMAND: EffectType
DELETE_PRESCRIBE_COMMAND: EffectType
COMMIT_PRESCRIBE_COMMAND: EffectType
ENTER_IN_ERROR_PRESCRIBE_COMMAND: EffectType
ORIGINATE_QUESTIONNAIRE_COMMAND: EffectType
EDIT_QUESTIONNAIRE_COMMAND: EffectType
DELETE_QUESTIONNAIRE_COMMAND: EffectType
COMMIT_QUESTIONNAIRE_COMMAND: EffectType
ENTER_IN_ERROR_QUESTIONNAIRE_COMMAND: EffectType
ORIGINATE_REASON_FOR_VISIT_COMMAND: EffectType
EDIT_REASON_FOR_VISIT_COMMAND: EffectType
DELETE_REASON_FOR_VISIT_COMMAND: EffectType
COMMIT_REASON_FOR_VISIT_COMMAND: EffectType
ENTER_IN_ERROR_REASON_FOR_VISIT_COMMAND: EffectType
ORIGINATE_STOP_MEDICATION_COMMAND: EffectType
EDIT_STOP_MEDICATION_COMMAND: EffectType
DELETE_STOP_MEDICATION_COMMAND: EffectType
COMMIT_STOP_MEDICATION_COMMAND: EffectType
ENTER_IN_ERROR_STOP_MEDICATION_COMMAND: EffectType
ORIGINATE_UPDATE_GOAL_COMMAND: EffectType
EDIT_UPDATE_GOAL_COMMAND: EffectType
DELETE_UPDATE_GOAL_COMMAND: EffectType
COMMIT_UPDATE_GOAL_COMMAND: EffectType
ENTER_IN_ERROR_UPDATE_GOAL_COMMAND: EffectType
ORIGINATE_PERFORM_COMMAND: EffectType
EDIT_PERFORM_COMMAND: EffectType
DELETE_PERFORM_COMMAND: EffectType
COMMIT_PERFORM_COMMAND: EffectType
ENTER_IN_ERROR_PERFORM_COMMAND: EffectType
ORIGINATE_INSTRUCT_COMMAND: EffectType
EDIT_INSTRUCT_COMMAND: EffectType
DELETE_INSTRUCT_COMMAND: EffectType
COMMIT_INSTRUCT_COMMAND: EffectType
ENTER_IN_ERROR_INSTRUCT_COMMAND: EffectType
ORIGINATE_LAB_ORDER_COMMAND: EffectType
EDIT_LAB_ORDER_COMMAND: EffectType
DELETE_LAB_ORDER_COMMAND: EffectType
COMMIT_LAB_ORDER_COMMAND: EffectType
ENTER_IN_ERROR_LAB_ORDER_COMMAND: EffectType
ORIGINATE_FAMILY_HISTORY_COMMAND: EffectType
EDIT_FAMILY_HISTORY_COMMAND: EffectType
DELETE_FAMILY_HISTORY_COMMAND: EffectType
COMMIT_FAMILY_HISTORY_COMMAND: EffectType
ENTER_IN_ERROR_FAMILY_HISTORY_COMMAND: EffectType
ORIGINATE_ALLERGY_COMMAND: EffectType
EDIT_ALLERGY_COMMAND: EffectType
DELETE_ALLERGY_COMMAND: EffectType
COMMIT_ALLERGY_COMMAND: EffectType
ENTER_IN_ERROR_ALLERGY_COMMAND: EffectType
ORIGINATE_REMOVE_ALLERGY_COMMAND: EffectType
EDIT_REMOVE_ALLERGY_COMMAND: EffectType
DELETE_REMOVE_ALLERGY_COMMAND: EffectType
COMMIT_REMOVE_ALLERGY_COMMAND: EffectType
ENTER_IN_ERROR_REMOVE_ALLERGY_COMMAND: EffectType
ORIGINATE_SURGICAL_HISTORY_COMMAND: EffectType
EDIT_SURGICAL_HISTORY_COMMAND: EffectType
DELETE_SURGICAL_HISTORY_COMMAND: EffectType
COMMIT_SURGICAL_HISTORY_COMMAND: EffectType
ENTER_IN_ERROR_SURGICAL_HISTORY_COMMAND: EffectType
CREATE_TASK: EffectType
UPDATE_TASK: EffectType
CREATE_TASK_COMMENT: EffectType
ORIGINATE_MEDICAL_HISTORY_COMMAND: EffectType
EDIT_MEDICAL_HISTORY_COMMAND: EffectType
DELETE_MEDICAL_HISTORY_COMMAND: EffectType
COMMIT_MEDICAL_HISTORY_COMMAND: EffectType
ENTER_IN_ERROR_MEDICAL_HISTORY_COMMAND: EffectType
ADD_OR_UPDATE_PROTOCOL_CARD: EffectType
ORIGINATE_TASK_COMMAND: EffectType
EDIT_TASK_COMMAND: EffectType
DELETE_TASK_COMMAND: EffectType
COMMIT_TASK_COMMAND: EffectType
ENTER_IN_ERROR_TASK_COMMAND: EffectType
ORIGINATE_REFILL_COMMAND: EffectType
EDIT_REFILL_COMMAND: EffectType
DELETE_REFILL_COMMAND: EffectType
COMMIT_REFILL_COMMAND: EffectType
ENTER_IN_ERROR_REFILL_COMMAND: EffectType
ORIGINATE_VITALS_COMMAND: EffectType
EDIT_VITALS_COMMAND: EffectType
DELETE_VITALS_COMMAND: EffectType
COMMIT_VITALS_COMMAND: EffectType
ENTER_IN_ERROR_VITALS_COMMAND: EffectType
ORIGINATE_UPDATE_DIAGNOSIS_COMMAND: EffectType
EDIT_UPDATE_DIAGNOSIS_COMMAND: EffectType
DELETE_UPDATE_DIAGNOSIS_COMMAND: EffectType
COMMIT_UPDATE_DIAGNOSIS_COMMAND: EffectType
ENTER_IN_ERROR_UPDATE_DIAGNOSIS_COMMAND: EffectType
ORIGINATE_CLOSE_GOAL_COMMAND: EffectType
EDIT_CLOSE_GOAL_COMMAND: EffectType
DELETE_CLOSE_GOAL_COMMAND: EffectType
COMMIT_CLOSE_GOAL_COMMAND: EffectType
ENTER_IN_ERROR_CLOSE_GOAL_COMMAND: EffectType
ANNOTATE_PATIENT_CHART_CONDITION_RESULTS: EffectType
ANNOTATE_CLAIM_CONDITION_RESULTS: EffectType
SHOW_PATIENT_CHART_SUMMARY_SECTIONS: EffectType
SHOW_PATIENT_PROFILE_SECTIONS: EffectType
PATIENT_PROFILE__ADD_PHARMACY__POST_SEARCH_RESULTS: EffectType
SEND_SURESCRIPTS_ELIGIBILITY_REQUEST: EffectType
SEND_SURESCRIPTS_MEDICATION_HISTORY_REQUEST: EffectType
SEND_SURESCRIPTS_BENEFITS_REQUEST: EffectType
ORIGINATE_EXAM_COMMAND: EffectType
EDIT_EXAM_COMMAND: EffectType
DELETE_EXAM_COMMAND: EffectType
COMMIT_EXAM_COMMAND: EffectType
ENTER_IN_ERROR_EXAM_COMMAND: EffectType
ORIGINATE_ROS_COMMAND: EffectType
EDIT_ROS_COMMAND: EffectType
DELETE_ROS_COMMAND: EffectType
COMMIT_ROS_COMMAND: EffectType
ENTER_IN_ERROR_ROS_COMMAND: EffectType
ORIGINATE_STRUCTURED_ASSESSMENT_COMMAND: EffectType
EDIT_STRUCTURED_ASSESSMENT_COMMAND: EffectType
DELETE_STRUCTURED_ASSESSMENT_COMMAND: EffectType
COMMIT_STRUCTURED_ASSESSMENT_COMMAND: EffectType
ENTER_IN_ERROR_STRUCTURED_ASSESSMENT_COMMAND: EffectType
SHOW_ACTION_BUTTON: EffectType
PATIENT_PORTAL__INTAKE_FORM_RESULTS: EffectType
LAUNCH_MODAL: EffectType

class Effect(_message.Message):
    __slots__ = ("type", "payload", "plugin_name", "classname")
    TYPE_FIELD_NUMBER: _ClassVar[int]
    PAYLOAD_FIELD_NUMBER: _ClassVar[int]
    PLUGIN_NAME_FIELD_NUMBER: _ClassVar[int]
    CLASSNAME_FIELD_NUMBER: _ClassVar[int]
    type: EffectType
    payload: str
    plugin_name: str
    classname: str
    def __init__(self, type: _Optional[_Union[EffectType, str]] = ..., payload: _Optional[str] = ..., plugin_name: _Optional[str] = ..., classname: _Optional[str] = ...) -> None: ...
